@@ -1,7 +1,7 @@
 /*
  * Seahorse
  *
- * Copyright (C) 2006 Nate Nielsen
+ * Copyright (C) 2006 Stefan Walter
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@ enum {
 
 struct _SeahorseGKeyringSourcePrivate {
     SeahorseMultiOperation *operations;     /* A list of all current operations */    
-    gchar *keyring_name;                    /* The keyring name */
+    gchar *keyring_name;                    /* The key ring name */
 };
 
 G_DEFINE_TYPE (SeahorseGKeyringSource, seahorse_gkeyring_source, SEAHORSE_TYPE_KEY_SOURCE);
@@ -85,6 +85,12 @@ static void         key_destroyed           (SeahorseKey *skey, SeahorseKeySourc
 #define SEAHORSE_IS_LIST_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SEAHORSE_TYPE_LIST_OPERATION))
 #define SEAHORSE_LIST_OPERATION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SEAHORSE_TYPE_LIST_OPERATION, SeahorseListOperationClass))
 
+enum {
+    PART_INFO = 0x01,
+    PART_ATTRS = 0x02,
+    PART_ACL = 0x04
+};
+
 DECLARE_OPERATION (List, list)
 
     SeahorseGKeyringSource *gsrc;
@@ -92,12 +98,16 @@ DECLARE_OPERATION (List, list)
     GList *remaining;
     guint total;
     
+    guint32 current_parts;
     guint32 current_id;
     guint32 info_flags;
     GnomeKeyringItemInfo *current_info;
     GnomeKeyringAttributeList *current_attrs;
+    GList *current_acl;
     
     gpointer request;
+    gpointer request_attrs;
+    gpointer request_acl;
     
     GHashTable *checks;
 
@@ -142,7 +152,17 @@ keyring_operation_failed (SeahorseListOperation *lop, GnomeKeyringResult result)
     if (!seahorse_operation_is_running (op))
         return;
     
-    g_assert (lop->request == NULL);
+    if (lop->request)
+        gnome_keyring_cancel_request (lop->request);
+    lop->request = NULL;
+
+    if (lop->request_attrs)
+        gnome_keyring_cancel_request (lop->request_attrs);
+    lop->request_attrs = NULL;
+
+    if (lop->request_acl)
+        gnome_keyring_cancel_request (lop->request_acl);
+    lop->request_acl = NULL;
     
     seahorse_gkeyring_operation_parse_error (result, &err);
     g_assert (err != NULL);
@@ -150,6 +170,60 @@ keyring_operation_failed (SeahorseListOperation *lop, GnomeKeyringResult result)
     seahorse_operation_mark_done (op, FALSE, err);
 }
 
+static gboolean
+have_complete_item (SeahorseListOperation *lop)
+{
+    SeahorseGKeyringItem *git = NULL;
+    SeahorseGKeyringItem *prev;
+    GQuark keyid;
+    
+    g_assert (lop->current_id);
+    
+    if (lop->current_parts != (PART_INFO | PART_ATTRS | PART_ACL))
+        return FALSE;
+    
+    g_assert (lop->current_info);
+  
+    keyid = seahorse_gkeyring_item_get_cannonical (lop->current_id);
+    g_return_val_if_fail (keyid, FALSE);
+    
+    /* Mark this key as seen */
+    if (lop->checks)
+        g_hash_table_remove (lop->checks, GUINT_TO_POINTER (keyid));
+    
+    g_assert (SEAHORSE_IS_GKEYRING_SOURCE (lop->gsrc));
+    prev = SEAHORSE_GKEYRING_ITEM (seahorse_context_get_key (SCTX_APP (), 
+                                   SEAHORSE_KEY_SOURCE (lop->gsrc), keyid));
+
+    /* Check if we can just replace the key on the object */
+    if (prev != NULL) {
+        g_object_set (prev, "item-info", lop->current_info, NULL);
+        if (lop->current_attrs)
+            g_object_set (prev, "item-attributes", lop->current_attrs, NULL);
+        g_object_set (prev, "item-acl", lop->current_acl, NULL);
+		
+        lop->current_info = NULL;
+        lop->current_acl = NULL;
+        lop->current_attrs = NULL;
+        return TRUE;
+    }
+    
+    git = seahorse_gkeyring_item_new (SEAHORSE_KEY_SOURCE (lop->gsrc), lop->current_id, 
+                                      lop->current_info, lop->current_attrs, lop->current_acl);
+ 
+    /* We listen in to get notified of changes on this key */
+    g_signal_connect (git, "changed", G_CALLBACK (key_changed), SEAHORSE_KEY_SOURCE (lop->gsrc));
+    g_signal_connect (git, "destroy", G_CALLBACK (key_destroyed), SEAHORSE_KEY_SOURCE (lop->gsrc));
+
+    /* Add to context */ 
+    seahorse_context_take_key (SCTX_APP (), SEAHORSE_KEY (git));
+
+    lop->current_info = NULL;
+    lop->current_acl = NULL;
+    lop->current_attrs = NULL;
+    return TRUE;
+}
+
 static void 
 item_info_ready (GnomeKeyringResult result, GnomeKeyringItemInfo *info, 
                  SeahorseListOperation *lop)
@@ -168,7 +242,10 @@ item_info_ready (GnomeKeyringResult result, GnomeKeyringItemInfo *info,
     g_assert (!lop->current_info);
     
     lop->current_info = gnome_keyring_item_info_copy (info);
-    process_next_item (lop);
+    lop->current_parts |= PART_INFO;
+    
+    if (have_complete_item (lop))
+        process_next_item (lop);
 }
 
 static void 
@@ -178,7 +255,7 @@ item_attrs_ready (GnomeKeyringResult result, GnomeKeyringAttributeList *attribut
     if (result == GNOME_KEYRING_RESULT_CANCELLED)
         return;
     
-    lop->request = NULL;
+    lop->request_attrs = NULL;
 
     if (result != GNOME_KEYRING_RESULT_OK) {
         keyring_operation_failed (lop, result);
@@ -189,47 +266,33 @@ item_attrs_ready (GnomeKeyringResult result, GnomeKeyringAttributeList *attribut
     g_assert (!lop->current_attrs);
     
     lop->current_attrs = gnome_keyring_attribute_list_copy (attributes);
-    process_next_item (lop);
+    lop->current_parts |= PART_ATTRS;
+    
+    if (have_complete_item (lop))
+        process_next_item (lop);
 }
 
-static void
-have_complete_item (SeahorseListOperation *lop)
+static void 
+item_acl_ready (GnomeKeyringResult result, GList *acl, SeahorseListOperation *lop)
 {
-    SeahorseGKeyringItem *git = NULL;
-    SeahorseGKeyringItem *prev;
-    GQuark keyid;
+    if (result == GNOME_KEYRING_RESULT_CANCELLED)
+        return;
     
-    g_assert (lop->current_id);
-    g_assert (lop->current_info);
-  
-    keyid = seahorse_gkeyring_item_get_cannonical (lop->current_id);
-    g_return_if_fail (keyid);
-    
-    /* Mark this key as seen */
-    if (lop->checks)
-        g_hash_table_remove (lop->checks, GUINT_TO_POINTER (keyid));
-    
-    g_assert (SEAHORSE_IS_GKEYRING_SOURCE (lop->gsrc));
-    prev = SEAHORSE_GKEYRING_ITEM (seahorse_context_get_key (SCTX_APP (), 
-                                   SEAHORSE_KEY_SOURCE (lop->gsrc), keyid));
+    lop->request_acl = NULL;
 
-    /* Check if we can just replace the key on the object */
-    if (prev != NULL) {
-        g_object_set (prev, "item-info", lop->current_info, NULL);
-        if (lop->current_attrs)
-            g_object_set (prev, "item-attributes", lop->current_attrs, NULL);
+    if (result != GNOME_KEYRING_RESULT_OK) {
+        keyring_operation_failed (lop, result);
         return;
     }
     
-    git = seahorse_gkeyring_item_new (SEAHORSE_KEY_SOURCE (lop->gsrc), lop->current_id, 
-                                      lop->current_info, lop->current_attrs);
- 
-    /* We listen in to get notified of changes on this key */
-    g_signal_connect (git, "changed", G_CALLBACK (key_changed), SEAHORSE_KEY_SOURCE (lop->gsrc));
-    g_signal_connect (git, "destroy", G_CALLBACK (key_destroyed), SEAHORSE_KEY_SOURCE (lop->gsrc));
-
-    /* Add to context */ 
-    seahorse_context_take_key (SCTX_APP (), SEAHORSE_KEY (git));
+    g_assert (lop->current_id);
+    g_assert (!lop->current_acl);
+    
+    lop->current_acl = gnome_keyring_acl_copy (acl);
+    lop->current_parts |= PART_ACL;
+    
+    if (have_complete_item (lop))
+        process_next_item (lop);
 }
 
 /* Remove the given key from the context */
@@ -250,29 +313,16 @@ process_next_item (SeahorseListOperation *lop)
 {
     g_assert (lop);
     g_assert (SEAHORSE_IS_GKEYRING_SOURCE (lop->gsrc));
-    g_assert (lop->gsrc->pv->keyring_name != NULL);
 
     seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (lop), NULL, 
                                            lop->total - g_list_length (lop->remaining), 
                                            lop->total);
     
-    if (lop->current_id) {
-        
-        /* Info loaded, now attributes */
-        if (lop->current_info && !lop->current_attrs) {
-            g_assert (!lop->request);
-            lop->request = gnome_keyring_item_get_attributes (lop->gsrc->pv->keyring_name, 
-                lop->current_id, (GnomeKeyringOperationGetAttributesCallback)item_attrs_ready, lop, NULL);
-            return;
-        }
-
-        g_assert (lop->current_info && lop->current_attrs);
-        have_complete_item (lop);
-        
-        lop->current_id = 0;
-        lop->current_info = NULL;
-        lop->current_attrs = NULL;
-    }
+   lop->current_id = 0;
+   lop->current_parts = 0;
+   g_assert (!lop->current_info);
+   g_assert (!lop->current_attrs);
+   g_assert (!lop->current_acl);
     
     /* Check if we're done */
     if (g_list_length (lop->remaining) == 0) {
@@ -292,18 +342,16 @@ process_next_item (SeahorseListOperation *lop)
     
     g_assert (!lop->request);
 
-	/* 
-	 * Check for the new gnome_keyring_item_get_info_full() function. 
- 	 * TODO: Once this is in a stable release of gnome-keyring, remove defs.
- 	 */
-#ifdef GNOME_KEYRING_ITEM_INFO_ALL
+    /* The various callbacks agree on when the item is fully loaded */
     lop->request = gnome_keyring_item_get_info_full (lop->gsrc->pv->keyring_name, 
                                 lop->current_id, lop->info_flags, 
                                 (GnomeKeyringOperationGetItemInfoCallback)item_info_ready, lop, NULL);
-#else
-    lop->request = gnome_keyring_item_get_info (lop->gsrc->pv->keyring_name, 
-                lop->current_id, (GnomeKeyringOperationGetItemInfoCallback)item_info_ready, lop, NULL);
-#endif
+
+    lop->request_attrs = gnome_keyring_item_get_attributes (lop->gsrc->pv->keyring_name, lop->current_id, 
+                                (GnomeKeyringOperationGetAttributesCallback)item_attrs_ready, lop, NULL);
+	                            
+    lop->request_acl = gnome_keyring_item_get_acl (lop->gsrc->pv->keyring_name, lop->current_id, 
+                                (GnomeKeyringOperationGetListCallback)item_acl_ready, lop, NULL);
 }
 
 static void 
@@ -333,7 +381,6 @@ start_list_operation (SeahorseGKeyringSource *gsrc, GQuark keyid)
     GList *keys, *l;
 
     g_assert (SEAHORSE_IS_GKEYRING_SOURCE (gsrc));
-    g_assert (gsrc->pv->keyring_name);
 
     lop = g_object_new (SEAHORSE_TYPE_LIST_OPERATION, NULL);
     lop->gsrc = gsrc;
@@ -349,11 +396,9 @@ start_list_operation (SeahorseGKeyringSource *gsrc, GQuark keyid)
         
         lop->remaining = g_list_prepend (lop->remaining, GUINT_TO_POINTER (id));
         lop->total = 1;
-
-#ifdef GNOME_KEYRING_ITEM_INFO_ALL
-        /* TODO: Once this is in a stable release of gnome-keyring, remove defs. */
+        
+        /* Load everything including the secret */
         lop->info_flags = GNOME_KEYRING_ITEM_INFO_ALL;
-#endif
         
         seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), _("Retrieving key"), -1);
         process_next_item (lop);
@@ -414,6 +459,9 @@ seahorse_list_operation_finalize (GObject *gobject)
     if (lop->current_attrs)
         gnome_keyring_attribute_list_free (lop->current_attrs);
     lop->current_attrs = NULL;
+    if (lop->current_acl)
+        gnome_keyring_acl_free (lop->current_acl);
+    lop->current_acl = NULL;
     
     if (lop->checks)
         g_hash_table_destroy (lop->checks);
@@ -462,31 +510,6 @@ key_destroyed (SeahorseKey *skey, SeahorseKeySource *sksrc)
     g_signal_handlers_disconnect_by_func (skey, key_destroyed, sksrc);
 }
 
-static gboolean
-init_keyring_name (SeahorseGKeyringSource *gsrc, GError **err)
-{
-    GnomeKeyringResult res;
-    gboolean ret;
-    
-    if (gsrc->pv->keyring_name)
-        return TRUE;
-
-    /* Get default keyring name when no other keyring has been specified */
-    res = gnome_keyring_get_default_keyring_sync (&(gsrc->pv->keyring_name));
-    if (res != GNOME_KEYRING_RESULT_OK)
-        g_warning ("couldn't get default gnome-keyring keyring: (code %d)", res);
-    
-    ret = seahorse_gkeyring_operation_parse_error (res, err);
-    if(!ret)
-        return FALSE;
-    
-    /* Happens when user has not yet initialized a keyring */
-    if (!gsrc->pv->keyring_name)
-        g_message ("no default gnome-keyring keyring found");
-
-    return TRUE;
-}
-
 /* -----------------------------------------------------------------------------
  * OBJECT 
  */
@@ -503,8 +526,6 @@ static GObject*
 seahorse_gkeyring_source_constructor (GType type, guint n_props, GObjectConstructParam* props)
 {
     GObject* obj = G_OBJECT_CLASS (seahorse_gkeyring_source_parent_class)->constructor (type, n_props, props);
-
-    init_keyring_name (SEAHORSE_GKEYRING_SOURCE (obj), NULL);
     return obj;
 }
 
@@ -551,18 +572,10 @@ seahorse_gkeyring_source_load (SeahorseKeySource *src, GQuark keyid)
 {
     SeahorseGKeyringSource *gsrc;
     SeahorseListOperation *lop;
-    GError *err = NULL;
     
     g_assert (SEAHORSE_IS_KEY_SOURCE (src));
     gsrc = SEAHORSE_GKEYRING_SOURCE (src);
     
-    if (!init_keyring_name (gsrc, &err))
-        return seahorse_operation_new_complete (err);
-
-    /* No keyring, act as if no items */
-    if (!gsrc->pv->keyring_name) 
-        return seahorse_operation_new_complete (NULL);
-
     lop = start_list_operation (gsrc, keyid);
     seahorse_multi_operation_take (gsrc->pv->operations, SEAHORSE_OPERATION (lop));
     
@@ -634,13 +647,6 @@ seahorse_gkeyring_source_remove (SeahorseKeySource *sksrc, SeahorseKey *skey,
     /* Don't have multiple names */
     g_return_val_if_fail (name == 0, FALSE);
    
-    if (!init_keyring_name (gsrc, err))
-        return FALSE;
-
-    /* No keyring, act as if deleted */
-    if (!gsrc->pv->keyring_name) 
-        return TRUE;
-    
     res = gnome_keyring_item_delete_sync (gsrc->pv->keyring_name, git->item_id);
     ret = seahorse_gkeyring_operation_parse_error (res, err);
     
@@ -748,6 +754,5 @@ seahorse_gkeyring_source_new (const gchar *keyring_name)
 const gchar*
 seahorse_gkeyring_source_get_keyring_name (SeahorseGKeyringSource *gsrc)
 {
-    init_keyring_name (gsrc, NULL);
     return gsrc->pv->keyring_name;
 }
